@@ -1,11 +1,11 @@
 from abc import abstractmethod
 from collections import OrderedDict
+from typing import Any
 from typing import Dict
 from typing import Iterator
 from typing import Optional
 from typing import Tuple
 from typing import TypeVar
-from typing import Union
 from typing import no_type_check
 
 import torch
@@ -15,12 +15,12 @@ from pfhedge._utils.doc import _set_attr_and_docstring
 from pfhedge._utils.doc import _set_docstring
 from pfhedge._utils.typing import TensorOrScalar
 
-from ..base import Instrument
+from ..base import BaseInstrument
 
-T = TypeVar("T", bound="Primary")
+T = TypeVar("T", bound="BasePrimary")
 
 
-class Primary(Instrument):
+class BasePrimary(BaseInstrument):
     """Base class for all primary instruments.
 
     A primary instrument is a basic financial instrument which is traded on a market
@@ -28,7 +28,7 @@ class Primary(Instrument):
     Examples include stocks, bonds, commodities, and currencies.
 
     Derivatives are issued based on primary instruments
-    (See :class:`Derivative` for details).
+    (See :class:`BaseDerivative` for details).
 
     Buffers:
         - spot (:class:`torch.Tensor`): The spot price of the instrument.
@@ -41,18 +41,18 @@ class Primary(Instrument):
 
     dt: float
     cost: float
-    _buffers: Dict[str, Optional[Tensor]]
+    _buffers: Dict[str, Tensor]
     dtype: Optional[torch.dtype]
     device: Optional[torch.device]
 
     def __init__(self) -> None:
         super().__init__()
         self._buffers = OrderedDict()
-        self.register_buffer("spot", None)
 
     @property
     def default_init_state(self) -> Tuple[TensorOrScalar, ...]:
         """Returns the default initial state of simulation."""
+        return ()
 
     # TODO(simaki): Remove @no_type_check once BrownianStock and HestonStock get
     #   unified signatures.
@@ -79,21 +79,21 @@ class Primary(Instrument):
                 (See :attr:`default_init_state`).
         """
 
-    def register_buffer(self, name: str, tensor: Optional[Tensor]) -> None:
+    def register_buffer(self, name: str, tensor: Tensor) -> None:
         """Adds a buffer to the instrument.
         The dtype and device of the buffer are the instrument's dtype and device.
 
         Buffers can be accessed as attributes using given names.
 
         Args:
-            name (string): name of the buffer. The buffer can be accessed
-                from this module using the given name
+            name (str): name of the buffer. The buffer can be accessed
+                from this module using the given name.
             tensor (Tensor or None): buffer to be registered. If ``None``, then
                 operations that run on buffers, such as :attr:`cuda`, are ignored.
         """
         # Implementation here refers to torch.nn.Module.register_buffer.
         if "_buffers" not in self.__dict__:
-            raise AttributeError("cannot assign buffer before Primary.__init__() call")
+            raise AttributeError("cannot assign buffer before __init__() call")
         elif not isinstance(name, torch._six.string_classes):
             raise TypeError(
                 "buffer name should be a string. " "Got {}".format(torch.typename(name))
@@ -126,7 +126,7 @@ class Primary(Instrument):
                 yield name, buffer
 
     def buffers(self) -> Iterator[Tensor]:
-        r"""Returns an iterator over module buffers.
+        """Returns an iterator over module buffers.
 
         Yields:
             torch.Tensor: module buffer
@@ -134,14 +134,22 @@ class Primary(Instrument):
         for _, buffer in self.named_buffers():
             yield buffer
 
-    def __getattr__(self, name: str) -> Tensor:
+    def get_buffer(self, name: str) -> Tensor:
+        """Returns the buffer given by target if it exists, otherwise throws an error.
+
+        Args:
+            name (str): the name of the buffer.
+
+        Returns:
+            torch.Tensor
+        """
         if "_buffers" in self.__dict__:
-            _buffers = self.__dict__["_buffers"]
-            if name in _buffers:
-                return _buffers[name]
-        raise AttributeError(
-            "'{}' object has no attribute '{}'".format(type(self).__name__, name)
-        )
+            if name in self._buffers:
+                return self._buffers[name]
+        raise AttributeError(self._get_name() + " has no buffer named " + name)
+
+    def __getattr__(self, name: str) -> Tensor:
+        return self.get_buffer(name)
 
     @property
     def spot(self) -> Tensor:
@@ -155,13 +163,16 @@ class Primary(Instrument):
             "Asset may not be simulated."
         )
 
-    def to(self: T, *args, **kwargs) -> T:
+    @property
+    def is_listed(self) -> bool:
+        return True
+
+    def to(self: T, *args: Any, **kwargs: Any) -> T:
         device, dtype, *_ = self._parse_to(*args, **kwargs)
 
         if dtype is not None and not dtype.is_floating_point:
             raise TypeError(
-                "Instrument.to only accepts floating point "
-                "dtypes, but got desired dtype=" + str(dtype)
+                f"to() only accepts floating point dtypes, but got desired dtype={dtype}"
             )
 
         if not hasattr(self, "dtype") or dtype is not None:
@@ -175,13 +186,13 @@ class Primary(Instrument):
         return self
 
     @staticmethod
-    def _parse_to(*args, **kwargs):
+    def _parse_to(*args: Any, **kwargs: Any):
         # Can be called as:
         #   to(device=None, dtype=None)
         #   to(tensor)
         #   to(instrument)
         # and return a tuple (device, dtype, ...)
-        if len(args) > 0 and isinstance(args[0], Instrument):
+        if len(args) > 0 and isinstance(args[0], BaseInstrument):
             instrument = args[0]
             return (getattr(instrument, "device"), getattr(instrument, "dtype"))
         elif "instrument" in kwargs:
@@ -200,10 +211,16 @@ class Primary(Instrument):
         return main_str
 
 
+class Primary(BasePrimary):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        raise DeprecationWarning("Primary is deprecated. Use BasePrimary instead.")
+
+
 # Assign docstrings so they appear in Sphinx documentation
-_set_docstring(Primary, "to", Instrument.to)
-_set_attr_and_docstring(Primary, "cpu", Instrument.cpu)
-_set_attr_and_docstring(Primary, "cuda", Instrument.cuda)
-_set_attr_and_docstring(Primary, "double", Instrument.double)
-_set_attr_and_docstring(Primary, "float", Instrument.float)
-_set_attr_and_docstring(Primary, "half", Instrument.half)
+_set_docstring(BasePrimary, "to", BaseInstrument.to)
+_set_attr_and_docstring(BasePrimary, "cpu", BaseInstrument.cpu)
+_set_attr_and_docstring(BasePrimary, "cuda", BaseInstrument.cuda)
+_set_attr_and_docstring(BasePrimary, "double", BaseInstrument.double)
+_set_attr_and_docstring(BasePrimary, "float", BaseInstrument.float)
+_set_attr_and_docstring(BasePrimary, "half", BaseInstrument.half)
