@@ -1,16 +1,22 @@
+from typing import Optional
+
 import torch
 from torch import Tensor
-from torch.distributions.utils import broadcast_all
 
 from pfhedge._utils.bisect import find_implied_volatility
 from pfhedge._utils.doc import _set_attr_and_docstring
 from pfhedge._utils.str import _format_float
-from pfhedge.nn.functional import d1
-from pfhedge.nn.functional import d2
-from pfhedge.nn.functional import ncdf
-from pfhedge.nn.functional import npdf
+from pfhedge.instruments import EuropeanOption
+from pfhedge.nn.functional import bs_european_delta
+from pfhedge.nn.functional import bs_european_gamma
+from pfhedge.nn.functional import bs_european_price
+from pfhedge.nn.functional import bs_european_theta
+from pfhedge.nn.functional import bs_european_vega
 
 from ._base import BSModuleMixin
+from ._base import acquire_params_from_derivative_0
+from ._base import acquire_params_from_derivative_1
+from .black_scholes import BlackScholesModuleFactory
 
 
 class BSEuropeanOption(BSModuleMixin):
@@ -55,10 +61,16 @@ class BSEuropeanOption(BSModuleMixin):
                 [0.5752]])
     """
 
-    def __init__(self, call: bool = True, strike: float = 1.0):
+    def __init__(
+        self,
+        call: bool = True,
+        strike: float = 1.0,
+        derivative: Optional[EuropeanOption] = None,
+    ) -> None:
         super().__init__()
         self.call = call
         self.strike = strike
+        self.derivative = derivative
 
     @classmethod
     def from_derivative(cls, derivative):
@@ -80,7 +92,9 @@ class BSEuropeanOption(BSModuleMixin):
             >>> m
             BSEuropeanOption(call=False, strike=1.)
         """
-        return cls(call=derivative.call, strike=derivative.strike)
+        return cls(
+            call=derivative.call, strike=derivative.strike, derivative=derivative
+        )
 
     def extra_repr(self) -> str:
         params = []
@@ -90,14 +104,17 @@ class BSEuropeanOption(BSModuleMixin):
         return ", ".join(params)
 
     def delta(
-        self, log_moneyness: Tensor, time_to_maturity: Tensor, volatility: Tensor
+        self,
+        log_moneyness: Optional[Tensor] = None,
+        time_to_maturity: Optional[Tensor] = None,
+        volatility: Optional[Tensor] = None,
     ) -> Tensor:
         """Returns delta of the derivative.
 
         Args:
-            log_moneyness (torch.Tensor): Log moneyness of the underlying asset.
-            time_to_maturity (torch.Tensor): Time to expiry of the option.
-            volatility (torch.Tensor): Volatility of the underlying asset.
+            log_moneyness (torch.Tensor, optional): Log moneyness of the underlying asset.
+            time_to_maturity (torch.Tensor, optional): Time to expiry of the option.
+            volatility (torch.Tensor, optional): Volatility of the underlying asset.
 
         Shape:
             - log_moneyness: :math:`(N, *)` where
@@ -108,23 +125,36 @@ class BSEuropeanOption(BSModuleMixin):
 
         Returns:
             torch.Tensor
+
+        Note:
+            Parameters are not optional if the module has not accepted a derivative in its initialization.
         """
-        s, t, v = broadcast_all(log_moneyness, time_to_maturity, volatility)
-
-        delta = ncdf(d1(s, t, v))
-        delta = delta - 1 if not self.call else delta  # put-call parity
-
-        return delta
+        (
+            log_moneyness,
+            time_to_maturity,
+            volatility,
+        ) = acquire_params_from_derivative_1(
+            self.derivative, log_moneyness, time_to_maturity, volatility
+        )
+        return bs_european_delta(
+            log_moneyness=log_moneyness,
+            time_to_maturity=time_to_maturity,
+            volatility=volatility,
+            call=self.call,
+        )
 
     def gamma(
-        self, log_moneyness: Tensor, time_to_maturity: Tensor, volatility: Tensor
+        self,
+        log_moneyness: Optional[Tensor] = None,
+        time_to_maturity: Optional[Tensor] = None,
+        volatility: Optional[Tensor] = None,
     ) -> Tensor:
         """Returns gamma of the derivative.
 
         Args:
-            log_moneyness: (torch.Tensor): Log moneyness of the underlying asset.
-            time_to_maturity (torch.Tensor): Time to expiry of the option.
-            volatility (torch.Tensor): Volatility of the underlying asset.
+            log_moneyness (torch.Tensor, optional): Log moneyness of the underlying asset.
+            time_to_maturity (torch.Tensor, optional): Time to expiry of the option.
+            volatility (torch.Tensor, optional): Volatility of the underlying asset.
 
         Shape:
             - log_moneyness: :math:`(N, *)` where
@@ -135,23 +165,36 @@ class BSEuropeanOption(BSModuleMixin):
 
         Returns:
             torch.Tensor
+
+        Note:
+            args are not optional if it doesn't accept derivative in this initialization.
         """
-
-        s, t, v = broadcast_all(log_moneyness, time_to_maturity, volatility)
-        price = self.strike * s.exp()
-        gamma = npdf(d1(s, t, v)) / (price * v * t.sqrt())
-
-        return gamma
+        (
+            log_moneyness,
+            time_to_maturity,
+            volatility,
+        ) = acquire_params_from_derivative_1(
+            self.derivative, log_moneyness, time_to_maturity, volatility
+        )
+        return bs_european_gamma(
+            log_moneyness=log_moneyness,
+            time_to_maturity=time_to_maturity,
+            volatility=volatility,
+            strike=self.strike,
+        )
 
     def vega(
-        self, log_moneyness: Tensor, time_to_maturity: Tensor, volatility: Tensor
+        self,
+        log_moneyness: Optional[Tensor] = None,
+        time_to_maturity: Optional[Tensor] = None,
+        volatility: Optional[Tensor] = None,
     ) -> Tensor:
         """Returns vega of the derivative.
 
         Args:
-            log_moneyness: (torch.Tensor): Log moneyness of the underlying asset.
-            time_to_maturity (torch.Tensor): Time to expiry of the option.
-            volatility (torch.Tensor): Volatility of the underlying asset.
+            log_moneyness (torch.Tensor, optional): Log moneyness of the underlying asset.
+            time_to_maturity (torch.Tensor, optional): Time to expiry of the option.
+            volatility (torch.Tensor, optional): Volatility of the underlying asset.
 
         Shape:
             - log_moneyness: :math:`(N, *)` where
@@ -162,22 +205,36 @@ class BSEuropeanOption(BSModuleMixin):
 
         Returns:
             torch.Tensor
-        """
-        s, t, v = broadcast_all(log_moneyness, time_to_maturity, volatility)
-        price = self.strike * s.exp()
-        vega = npdf(d1(s, t, v)) * price * t.sqrt()
 
-        return vega
+        Note:
+            args are not optional if it doesn't accept derivative in this initialization.
+        """
+        (
+            log_moneyness,
+            time_to_maturity,
+            volatility,
+        ) = acquire_params_from_derivative_1(
+            self.derivative, log_moneyness, time_to_maturity, volatility
+        )
+        return bs_european_vega(
+            log_moneyness=log_moneyness,
+            time_to_maturity=time_to_maturity,
+            volatility=volatility,
+            strike=self.strike,
+        )
 
     def theta(
-        self, log_moneyness: Tensor, time_to_maturity: Tensor, volatility: Tensor
+        self,
+        log_moneyness: Optional[Tensor] = None,
+        time_to_maturity: Optional[Tensor] = None,
+        volatility: Optional[Tensor] = None,
     ) -> Tensor:
         """Returns theta of the derivative.
 
         Args:
-            log_moneyness: (torch.Tensor): Log moneyness of the underlying asset.
-            time_to_maturity (torch.Tensor): Time to expiry of the option.
-            volatility (torch.Tensor): Volatility of the underlying asset.
+            log_moneyness (torch.Tensor, optional): Log moneyness of the underlying asset.
+            time_to_maturity (torch.Tensor, optional): Time to expiry of the option.
+            volatility (torch.Tensor, optional): Volatility of the underlying asset.
 
         Shape:
             - log_moneyness: :math:`(N, *)` where
@@ -191,21 +248,36 @@ class BSEuropeanOption(BSModuleMixin):
 
         Returns:
             torch.Tensor
+
+        Note:
+            args are not optional if it doesn't accept derivative in this initialization.
         """
-        s, t, v = broadcast_all(log_moneyness, time_to_maturity, volatility)
-        price = self.strike * s.exp()
-        theta = -npdf(d1(s, t, v)) * price * v / (2 * t.sqrt())
-        return theta
+        (
+            log_moneyness,
+            time_to_maturity,
+            volatility,
+        ) = acquire_params_from_derivative_1(
+            self.derivative, log_moneyness, time_to_maturity, volatility
+        )
+        return bs_european_theta(
+            log_moneyness=log_moneyness,
+            time_to_maturity=time_to_maturity,
+            volatility=volatility,
+            strike=self.strike,
+        )
 
     def price(
-        self, log_moneyness: Tensor, time_to_maturity: Tensor, volatility: Tensor
+        self,
+        log_moneyness: Optional[Tensor] = None,
+        time_to_maturity: Optional[Tensor] = None,
+        volatility: Optional[Tensor] = None,
     ) -> Tensor:
         """Returns price of the derivative.
 
         Args:
-            log_moneyness (torch.Tensor): Log moneyness of the underlying asset.
-            time_to_maturity (torch.Tensor): Time to expiry of the option.
-            volatility (torch.Tensor): Volatility of the underlying asset.
+            log_moneyness (torch.Tensor, optional): Log moneyness of the underlying asset.
+            time_to_maturity (torch.Tensor, optional): Time to expiry of the option.
+            volatility (torch.Tensor, optional): Volatility of the underlying asset.
 
         Shape:
             - log_moneyness: :math:`(N, *)` where
@@ -216,31 +288,37 @@ class BSEuropeanOption(BSModuleMixin):
 
         Returns:
             torch.Tensor
+
+        Note:
+            args are not optional if it doesn't accept derivative in this initialization.
         """
-        s, t, v = broadcast_all(log_moneyness, time_to_maturity, volatility)
-
-        n1 = ncdf(d1(s, t, v))
-        n2 = ncdf(d2(s, t, v))
-
-        price = self.strike * (s.exp() * n1 - n2)
-
-        if not self.call:
-            price += self.strike * (1 - s.exp())  # put-call parity
-
-        return price
+        (
+            log_moneyness,
+            time_to_maturity,
+            volatility,
+        ) = acquire_params_from_derivative_1(
+            self.derivative, log_moneyness, time_to_maturity, volatility
+        )
+        return bs_european_price(
+            log_moneyness=log_moneyness,
+            time_to_maturity=time_to_maturity,
+            volatility=volatility,
+            strike=self.strike,
+            call=self.call,
+        )
 
     def implied_volatility(
         self,
-        log_moneyness: Tensor,
-        time_to_maturity: Tensor,
-        price: Tensor,
+        log_moneyness: Optional[Tensor] = None,
+        time_to_maturity: Optional[Tensor] = None,
+        price: Optional[Tensor] = None,
         precision: float = 1e-6,
     ) -> Tensor:
         """Returns implied volatility of the derivative.
 
         Args:
-            log_moneyness (torch.Tensor): Log moneyness of the underlying asset.
-            time_to_maturity (torch.Tensor): Time to expiry of the option.
+            log_moneyness (torch.Tensor, optional): Log moneyness of the underlying asset.
+            time_to_maturity (torch.Tensor, optional): Time to expiry of the option.
             price (torch.Tensor): Price of the derivative.
             precision (float): Computational precision of the
                 implied volatility.
@@ -254,7 +332,19 @@ class BSEuropeanOption(BSModuleMixin):
 
         Returns
             torch.Tensor
+
+        Note:
+            args are not optional if it doesn't accept derivative in this initialization.
+            price seems optional in typing, but it isn't. It is set for the compatibility to the previous versions.
         """
+        (log_moneyness, time_to_maturity) = acquire_params_from_derivative_0(
+            self.derivative, log_moneyness, time_to_maturity
+        )
+        if price is None:
+            raise ValueError(
+                "price is required in this method. None is set only for compatibility to the previous versions."
+            )
+
         return find_implied_volatility(
             self.price,
             price=price,
@@ -264,6 +354,9 @@ class BSEuropeanOption(BSModuleMixin):
         )
 
 
+factory = BlackScholesModuleFactory()
+factory.register_module("EuropeanOption", BSEuropeanOption)
+
 # Assign docstrings so they appear in Sphinx documentation
 _set_attr_and_docstring(BSEuropeanOption, "inputs", BSModuleMixin.inputs)
 _set_attr_and_docstring(BSEuropeanOption, "forward", BSModuleMixin.forward)
